/* If the timestep changes by a factor "base" and the
 * errors are e1 and e2, respectively, then the order
 * is log(e1/e2)/log(base)
 */

#include "adaptive_verlet.h"
#include <assert.h>
#include <math.h>

#define NUM_H 22

#define T_FINAL 1.0

void pendulum2(int n, double t, double *r, double *rpp)
{
    assert(n==1);
    rpp[0] = -r[0];
}

static double DT;

double *VecDiff(int n, double diff[n], double v1[n], double v2[n])
{
    int i;
    for(i=0; i<n; i++)
	diff[i] = v1[i] - v2[i];
    return diff;
}

double VecNorm(int n, double v[n])
{
    double norm = 0.0;
    int i;
    for(i=0; i<n; i++)
	norm += v[i]*v[i];
    return sqrt(norm);
}

double IntPow(double base, int n)
{
    double result = 1.0;
    int i;
    if(n < 0)
	return 1/IntPow(base, -n);

    for(i=0; i<n; i++)
	result *= base;
    return result;
}

int main(void)
{
    double base=2.0, ycheap[2], ygood[2], error[NUM_H];
    int i;
    
    i=0; /* base case, with largest timestep */
    DT = IntPow(base, -i);
    ycheap[0] = 1.0;
    ycheap[1] = 0.0;
    init_adaptive_verlet(1, 0.0, DT, ycheap, ycheap+1, pendulum2);
    assert(T_FINAL == integrate_adaptive_verlet(T_FINAL));

    for(i=1; i<NUM_H; i++)
    {
	double diff[2];

	DT = IntPow(base, -i);

	ygood[0] = 1.0;
	ygood[1] = 0.0;
	init_adaptive_verlet(1, 0.0, DT, ygood, ygood+1, pendulum2);
	assert(T_FINAL == integrate_adaptive_verlet(T_FINAL));

	VecDiff(2, diff, ygood, ycheap);
	error[i] = VecNorm(2, diff);
	if(i > 0)
	{
	    double order = log(error[i-1]/error[i])/log(base);
	    printf("i=%2d DT=%.9f error=%.6e order=%4f\n", i, DT, error[i], order);
	}

	VecCopy(2, ycheap, ygood);
    }

    return 0;
}
